var _ = require('lodash');
var async = require('async');
var md5 = require('md5');

module.exports = function() {
  return function myMiddleware(req, res, next) {
    const { app, accessToken } = req;
    const { zlSite, socketQuery } = res.locals;

    // set referrer cookie
    if (!app.locals.bypass(req) && !req.cookies.referer) {
      res.cookie('referer', req.get('Referrer'), { maxAge: 2592000000 });
    }

    if (!accessToken || app.locals.bypass(req, {files: 1})){
      // remove cookie if set
      if (!accessToken && req.signedCookies.authorization) {
        let opt = _cookieOpt(req, res);
        res.clearCookie('authorization', opt);
      }

      return next();
    }

    // so we can push udates to user profile to any site displaying it
    socketQuery['usesProfile'] = [accessToken.userId];

    let parallel = [];
    if (!req.query.simulating && !req.get('X-User-Simulating')) {
      // if not simulating, do these functions
      parallel.push(
        (cb) => tokenToCookie(req, res, cb),
        (cb) => tokenQueryRemove(req, res, cb)
      );
    }
    parallel.push((cb) => loadUsrInfo(accessToken, app, zlSite, cb));
    async.parallel(parallel, () => {
      if (req.query.simulating) {
        zlSite.user.simulating = true;
      }
      next();
    });
  }
};

/* Access token live time prolongation
@see: https://github.com/strongloop/loopback/issues/257
function prolong(token, next) {
  var now = new Date(),
      createTS = token ? token.created.getTime() : 0,
      expiredTS = token ? token.ttl * 1000 : 0,
      dayTS = 24 * 60 * 60 * 1000;

  // expired token
  if (createTS + expiredTS < now) return next();

  if (now.getTime() - createTS < dayTS) {
    // create today
    return next();
  }
  // update date created to now (extend expire date)
  token.updateAttribute('created', now, next);
}*/

/* although we don't rely much on cookies, we save last request auth in cookie
in case they refresh page */
function tokenToCookie(req, res, next) {
  if (req.signedCookies.authorization !== req.accessToken.id) {
    let opt = _cookieOpt(req, res);
    res.cookie('authorization', req.accessToken.id, opt);
  }
  next();
}

// removes token from URL query string (will already be in cookie)
function tokenQueryRemove(req, res, next) {
  if (req.query && req.query.access_token && !req.app.locals.bypass(req)) {
    // redirect to URL without token
    let stripUrl = '';
    _.forEach(req.query, function(val, key) {
      if ('access_token' !== key) {
        stripUrl = (stripUrl)? `${stripUrl}&${key}=${val}`: `?${key}=${val}`;
      }
    });
    res.redirect(req.path + stripUrl);
  }
  else return next();
}

// load user details into variable
function loadUsrInfo(token, app, zlSite, next) {
  app.models.zlUser.findById(token.userId, (err, usr) => {
    if (err) next(err);
    else if (usr) {
      zlSite.user = _.cloneDeep(usr);
      if (!zlSite.user.usrPhoto && zlSite.user.email) {
        // use gravatar
        let graHash = md5(zlSite.user.email.toLowerCase());
        zlSite.user.usrPhoto = `https://www.gravatar.com/avatar/${graHash}?s=100&d=mm`
      }
      next();
    }
    else {
      // user must no longer exist
      next();
    }
  });
}

function _cookieOpt(req, res) {
  let opt = { signed: true };
  /* make available to root domain (not only subs if using sub)
  so domains like zealder.com, api.zealder.com can share auth
  Will not work for multipart ext like co.uk
  TODO: may have to let admin-theme set for root domain */
  var domainParts = res.locals.zlSite.zsDomain.split('.').reverse();
  opt.domain = `${domainParts[1]}.${domainParts[0]}`;
  if (req.secure) opt.secure = true;

  return opt;
}
