// @flow
'use strict';

var _ = require('lodash');
var fs = require('fs');

// loads pages
function serverRender(req, res, next) {
  var app = req.app;

  var { themes, defaultTheme } = app.locals;
  var { zlSite } = res.locals;

  if (!zlSite || !themes[zlSite.zsTheme]) return next();

  // determine if to use servBundle
  if ('production' === app.get('env') && themes[zlSite.zsTheme].servBundle) {
    themes[zlSite.zsTheme].servBundle(req, res);
  }
  else {
    // not using servBundle, transpile in real-time
    require("babel-register");

    // find the 1st serverRenderFile in themeTree
    let rendered = false;
    _.forEach(zlSite.themeTree, function(v, key) {
      if (themes[key].zealderConfig && themes[key].zealderConfig.serverRenderFile) {
        var serverRender = require(`${themes[key].path}/${themes[key].zealderConfig.serverRenderFile}`);
        serverRender(req, res);
        rendered = true;
        return false;
      }
    });
    if (!rendered) {
      // if none of the themes in tree provided serverRenderFile, use default
      var serverRender = require(`${themes[defaultTheme].path}/${themes[defaultTheme].zealderConfig.serverRenderFile}`);
      serverRender(req, res);
    }
  }
};

module.exports = function() { return serverRender };
