// @flow
'use strict';

var _ = require('lodash');

function setup(req, res, next) {
  var zlSite = res.locals.zlSite;
  var { themes } = req.app.locals;

  if (!zlSite || !themes[zlSite.zsTheme]) return next();

  // add css class name for theme
  res.locals.htmlClass.push('theme-'+zlSite.zsTheme);

  // load themeConfig
  zlSite.themeConfig = themes[zlSite.zsTheme].zealderConfig;

  // build theme tree starting with active theme
  buildThemes(req, zlSite, next);
}

module.exports = function() {
  return setup;
};

function buildThemes(req, zlSite, cb) {
  var { cache, getInheritTheme } = req.app.locals;
  cache.get('themeTrees', zlSite.zsTheme, (err, data) => {
    if (data) {
      zlSite.themeTree = _.cloneDeep(JSON.parse(data));
      cb();
    }
    else {
      zlSite.themeTree = getInheritTheme(zlSite.zsTheme);
      req.app.debug('%s theme tree: %o', zlSite.zsTheme, zlSite.themeTree);
      let v =  JSON.stringify(zlSite.themeTree);
      cache.set('themeTrees', zlSite.zsTheme, v, () => {
        cb();
      });
    }
  });
}
