// @flow
'use strict';

var async = require("async");
var _ = require('lodash');

function middleware(req, res, next) {
  var zlSite = res.locals.zlSite;
  var app = req.app;
  var { themes, bypass, defaultTheme, apps } = app.locals;

  if (!zlSite || !themes[zlSite.zsTheme] || bypass(req, {files: 1})) {
    return next();
  }

  let series = [];
  // check all themes in tree
  _.forEach(zlSite.themeTree, function(v, key) {
    if (key === defaultTheme) {
      // and we dont bundle defaultTheme yet
      return false;
    }
    // get middleware (if any)
    addSeries(req, res, zlSite, themes[key], series);
  });

  // check all Apps
  if (zlSite.zsApps) {
    _.forEach(zlSite.zsApps, function(key) {
      if (apps[key]) {
        addSeries(req, res, zlSite, apps[key], series);
      }
    });
  }

  // lastly, default theme
  addSeries(req, res, zlSite, themes[defaultTheme], series);

  // now run in "series" if there are middleware
  if (0 < series.length) {
    async.series(series, () => next());
  }
  else next();
}

module.exports = function() {
  return middleware;
};

function addSeries(req, res, zlSite, pack, series) {
  // now run pack middleware if exist
  if (pack.middleware) {
    series.push(
      (next) => {
        pack.middleware(req, res, next);
      }
    );
  }

  // additional mounts provided by theme
  if (pack.serverMount) {
    series.push(
      (next) => {
        pack.serverMount(req, res, next);
      }
    );
  }
}
