//@flow

'use strict';

var _ = require('lodash');
var app = require('../../server/server');

module.exports = function(zlSiteContentIn) {
  zlSiteContentIn.contentsToDisplay = contentsToDisplay;
  _.forEach(['get', 'post'], function(val) {
    zlSiteContentIn.remoteMethod(
      'contentsToDisplay', {
        http: { path: '/contentsToDisplay', verb: val },
        accepts: [
          {arg: 'req', type: 'object', 'http': {source: 'req'}},
          {arg: 'res', type: 'object', 'http': {source: 'res'}},
          { arg: 'scsId', type: 'any', required: true, 'http': {source: 'query'}},
          { arg: 'scId', type: 'any', required: false, 'http': {source: 'query'}}
        ],
        returns: { arg: 'data', type: 'object' }
      }
    );
  });
};

// get contents to display on a page
function contentsToDisplay(req, res, scsId, scId = 0, cb) {
  var zlSiteContentIn = app.models.zlSiteContentIn;
  var Role = app.models.Role;
  var compMod = require('./helpers/component-from-model')(app);

  // emit that content was requested
  if (res.locals.emiter.emit(`fetching content ${scsId}`, cb)) {
    // if a package listen, it is responsible for callback
    // also enable using scsId that is not in db to not cause error
    return;
  }

  var { themeData } = res.locals;

  // first, make sure they are allowed to view page
  // get role permission for page
  let filter = {fields: {scsPermission: true, zlSiteId: true}};
  app.models.zlSiteContentSet.findById(scsId, filter, (err, scs) => {
    if (err || !scs) return cb(`page does not exist`);
    let ctx = app.locals.authCtx(req, res);
    Role.isInRole(scs.scsPermission, ctx, (err, isInRole) => {
      if (isInRole) {
        // filter for model search
        let filter = {
          fields: {scId: true},
          include: {
            relation: 'zlSiteContent',
            scope: {
              where: {scStatus: 'publish', zlSiteId: res.locals.zlSite.zsId},
            },
          },
          where: {scsId: scsId},
        };
        zlSiteContentIn.find(filter, (err, data) => {
          // to keep list of promises
          var promises = [];
          // functions in component-from-model may need to make promises
          compMod.promises = promises;

          _.forEach(data, function(val, key) {
            if (!val || !val.zlSiteContent) return data.splice(key, 1);
            // converts from loopback object to json
            val = val.toJSON().zlSiteContent;
            if ('page content' === val.scType) {
              // overide data to this value
              data = val.scContent;
              // iterate list of page content
              if (_.isArray(data)) {
                _.forEach(data, function(subVal, subKey) {
                  // make sure there is a scId
                  subVal.scId = (subVal.scId)?subVal.scId:`pg${scsId}-cont${subKey}`;

                  prepContent(req, res, data, subVal, subKey, promises,
                    compMod, scsId, cb);
                });
              }
              // end here since only 1 active content list is used
              return false;
            } else {
              // other type (like 'post' but not yet used anywhere)
              data[key] = val.scContent;
              prepContent(req, res, data, val.scContent, key, promises,
                compMod, scsId, cb);
            }
          });
          // emit that content loaded; allowing other sources to populate further
          res.locals.emiter.emit(`loaded content ${scsId}`, data, promises);
          // when all promise met, callback
          Promise.all(promises).then(values => {
            cb(null, data);
          });
        });
      } else cb('Page Authorization failed');
    });
  });

  compMod.themeData = themeData;
}

function prepContent(req, res, allVal, val = {}, key, promises = [],
  compMod={}, scsId, cb=()=>{}) {
  const Role = app.models.Role;
  const ctx = app.locals.authCtx(req, res);
  // some render code that eval use this short alias
  var themeData = res.locals.themeData;

  if (!val) return allVal.splice(key, 1);

  let scPermission = (val.scPermission)?val.scPermission:'Everyone';
  // prepare content for display
  let scContent = val.scContent;

  promises.push( new Promise((resolve, reject) => {
    // make sure they are allowed to view content
    Role.isInRole(scPermission, ctx, (err, isInRole) => {
      if (!isInRole) {
        allVal.splice(key, 1);
      }
      resolve();
    });
  }));

  // emit that a specific content type was requested
  if (res.locals.emiter.emit(`fetching scType ${val.scType}`, val, promises)) {
    // if a package listen, then it will handle server-side config
    return;
  }

  if (!scContent) return;

  // prepare content that interct with a model
  if (scContent['forModel']) {
    if (_.isEmpty(app.models[scContent['forModel']])) {
      // if model doesn't exist, just continue
      return;
    }

    // code for rendering on server
    if (scContent['serverRender']) {
      eval(scContent['serverRender']);
      delete scContent['serverRender'];
    }

    // default api to call when interacting with data
    scContent['api'] = `/api/${scContent['forModel']}s`;

    if ('Datatable' === val.scType) {
      // prepare content for datatable
      promises.push( new Promise((resolve, reject) => {
        // get data from model
        app.models[scContent['forModel']].find(
          scContent['filter'], (err, modelData) => {
            // load data from model
            scContent['data'] = compMod.loadData(modelData, scContent);
            // make columns
            scContent['columns'] = compMod.makeColumns(scContent);
            // remove unnessary data for security
            delete scContent['filter'];
            resolve();
        })
      }));
    }
    else if ('Form' === val.scType) {
      // prepare content for form
      // api to call when POST (back to self/here)
      scContent['apiPost'] = `/api/zlSiteContentIns/contentsToDisplay?`
        +`scsId=${scsId}&scId=${val.scId}`;

      promises.push( new Promise((resolve) => {
        if ('POST' === req.method && val.scId === req.query.scId) {
          // process post process
          let saveParam = [ res, scContent, req.body, (err, result) => {
            if (err) req.app.error('error: %O resulted from POST:%O', err, req.body);
            if (scContent.responseHtml) {
              // send responseHtml
              // TODO: we can map result data to var in responseHtml in future
              result = { responseHtml: scContent.responseHtml} ;
            }
            // send result
            return cb(null, result);
          }];
          // recaptcha veryfication required when user unknown
          req.app.locals.recaptchaVerify(req, res, scContent['forModel'], (err) => {
            if (err) {
              req.app.error(err);
              // pretend form was submitted
              cb(null, {responseHtml: 'Your form submission was successful!'});
            }
            else compMod.saveForm(...saveParam);
          });
        }
        else {
          // make fields list
          scContent['fields'] = compMod.makeFormFields(res, scContent);
          contentAfterPrep(scContent, resolve);
        }
      }));
    }
  }

  // prep children content
  if (scContent.children) {
    _.forEach(scContent.children, function(chVal, chKey) {
      prepContent(req, res, scContent.children, chVal, chKey, promises, compMod,
        scsId, cb);
    });
  }
}

function contentAfterPrep (scContent, cb = ()=>{}) {
  // code for rendering on server after data has prepared
  // NOTE: not really used anywhere yet
  if (scContent['serverRenderAfter']) {
    eval(scContent['serverRenderAfter']);
  }

  // remove eval and other data that shouldn't be in response
  delete scContent.serverRenderAfter;
  delete scContent.responseHtml;
  cb();
}
