var loopback = require('loopback');
var boot = require('loopback-boot');
var http = require('http');

var app = module.exports = loopback();

// app start function
app.start = function() {
  // launch Web and Socket.io server
  var server = http.createServer(app);
  app.io = require('socket.io')(server);
  require('../socket')(app);
  server.listen(app.get('port'), () => {
    var baseUrl = 'http://' + app.get('host') + ':' + app.get('port');
    app.logger.info('Web and Socket.io server listening at: %s', baseUrl);
  });
};

app.load = function(config = {}, cb = ()=>{}) {
  // load configs
  app.locals.config = config;

  console.log('Zealder is booting up...');
  // set cookieSecret
  if (app.locals.config.cookieSecret) {
    app.set('cookieSecret', app.locals.config.cookieSecret);
  }
  else {
    app.set('cookieSecret', 'no secret and not secure');
    console.error('no config.cookieSecret provided. Cookies is not secure');
  }

  let postBoot = () => {};
  if (process.env.WORK) {
    // run as a script worker (probabbly not good idea to use in production
    // when spawn as new process)
    postBoot = () => {
      let work = require(app.locals.zlScripts[process.env.WORK]);
      if (work) work(app);
    }
  }
  else {
    // run as a web server
    postBoot = () => {
      app.debug('booted zealder');
      cb();
    };
  }

  // Bootstrap the application
  boot(app, __dirname, function(err) {
    if (err) throw err;
    // wait for all pacages to load, then continue
    // because if we listen for request before all themes loaded,
    // only default will bundle
    app.on('zealder packages loaded', postBoot);
  });
};
