// @flow
'use strict';

/* ==========================================================================
   WARNING: NEVER BROADCAST SENSITIVE DATA BECAUSE ROOM MEMBERS ARE NOT
   AUTHENTICATED; IF SENSITIVE DETAILS MUST UPDATE, ONLY SEND A EVENT NAME
   AND ALLOW SITE TO REQUEST DATA AFTER RECEIVING EVENT
   TODO: use app to  call Role.isInRole to get their permission before assigning to rooms
   ========================================================================== */

var _ = require('lodash');
var events = require('./actions');

// setuo socket.io
module.exports = function (app) {
  // attach event types to app
  app.locals.ioEvents = events;

  const io = app.io;

  if (app.locals.config.cacheDb && 'redis' === app.locals.config.cacheDb.connector) {
    // use redis to allow multi-node (cluster mode) to work
    var redis = require('socket.io-redis');
    io.adapter(redis(app.locals.config.cacheDb));
  }
  else {
    app.error('No cacheDb.connector was provided. '+
    'This will cause error if using node in cluster mode.');
  }

  io.on("connect", function(socket) {
    // assign to general room for site
    if (socket.handshake.query.zsId) {
      const room = `site-${socket.handshake.query.zsId}`;
      socket.join(room);

      // listen for changes
      socket.on(events.PAGE_UPDATED, function(data) {
        // re-broadcast to all others on same site
        socket.to(room).emit(events.PAGE_UPDATED, data);
      });
      socket.on(events.EDIT_DATA_OF, function(data) {
        // broadcast to all in room including sender
        io.to(room).emit(events.EDIT_DATA_OF, data);
      });
    }

    // assign to room that possibly displays/uses a user profile
    if (socket.handshake.query.usesProfile) {
      const usesProfile = socket.handshake.query.usesProfile.split(",");
      _.forEach(usesProfile, function(usrId) {
        socket.join(`usesProfile-${usrId}`);
      });
    }

    // assign to room for managingSite
    if (socket.handshake.query.managingSite) {
      socket.join(`managingSite-${socket.handshake.query.managingSite}`);
    }

    // assign to room for authSites
    if (socket.handshake.query.authSites) {
      const authSites = socket.handshake.query.authSites.split(",");
      _.forEach(authSites, function(zsId) {
        let aSiteRoom = `authSite-${zsId}`;
        socket.join(aSiteRoom);
      });
    }
  });
}
