'use strict';

var wpFunc = require('../src/helpers/functions');
var _ = require('lodash');
var request = require('request');

function middleware(req, res, next) {
  const {emiter, zlSite, themeData} = res.locals;
  const {wpConfig} = zlSite;

  if (!wpConfig || !wpConfig.apiUrl || req.app.locals.bypass(req, {files: 1})) {
    return next();
  }

  // setup object for data for theme
  var wpViewConf = {
    apiUrl: wpConfig.apiUrl,
  };

  // listen for pages setup to determin if there are Blog pages
  /* NOTE: since 'pages-loaded' is not emtted on ajax request for content, blog
  post are never preloaded */
  emiter.on('pages-loaded', function(pages) {
    // get Blog page
    let page = _.find(pages, ['scsBase', 'Blog']);
    if (!_.isEmpty(page)) {
      // add route for detail page
      pages[`${page.scsId}-post`] = Object.assign({}, page, {
        path: `${page.path}/:slug`,
        scsBase: 'BlogPost',
        scsId: `${page.scsId}-post`,
        scsOptions: {},
      });

      // add route for pagination
      pages[`${page.scsId}-pager`] = Object.assign({}, page, {
        path: `${page.path}/page/:pageNum`,
        scsOptions: {},
      });

      // add route for categories
      pages[`${page.scsId}-topics`] = Object.assign({}, page, {
        path: `${page.path}/topics/:catSlug`,
        scsId: `${page.scsId}-topics`,
        scsOptions: {},
      });

      // push additional page's content
      emiter.on(`loaded content ${page.scsId}`, (data, promises) => {
        promises.push(new Promise(function(resolve) {
          // initial posts data
          let conf = {};
          // check for initial page
          let pageNum = req.path.match(/page\/([0-9])/);
          if (pageNum && pageNum[1]) {
            conf.page = pageNum[1];
          }
          wpFunc.fetchData(wpViewConf.apiUrl, '', conf, (err, val, headers) => {
            if (val) {
              data.push({
                scType: 'WpPosts',
                scId: 'wp-posts-index',
                scContent: {posts: val, headers: headers, conf: conf},
              });
            }
            resolve();
          });
        }));
      });
      // push single post data
      emiter.on(`loaded content ${page.scsId}-post`, (data, promises) => {
        promises.push(new Promise(function(resolve) {
          // initial posts data
          let conf = {};
          // check for initial page
          let slug = req.path.match(/\/([^/]+)$/);
          if (slug && slug[1]) {
            conf.slug = slug[1];
            wpFunc.fetchData(wpViewConf.apiUrl, 'post', conf, (err, val) => {
              if (val && val[0]) {
                data.push({
                  scContent: val[0],
                  scId: `post-${conf.slug}`,
                });
              }
              resolve();
            });
          } else resolve();
        }));
      });
      // push categories post data
      emiter.on(`loaded content ${page.scsId}-topics`, (data, promises) => {
        promises.push(new Promise(function(resolve) {
          // initial posts data
          let conf = {};
          // check for initial page
          let slug = req.path.match(/topics\/([^/]+)$/);
          if (slug && slug[1]) {
            // get slug category
            let cat = _.find(wpViewConf.categories, ['slug', slug[1]]);
            if (cat) {
              conf.categories = cat['id'];
              wpFunc.fetchData(wpViewConf.apiUrl, '', conf, (err, val, headers) => {
                if (val) {
                  data.push({
                    scType: 'WpPosts',
                    scId: 'wp-posts-index',
                    scContent: {posts: val, headers: headers, conf: conf},
                  });
                }
                resolve();
              });
            }
          } else resolve();
        }));
      });
    }
  });

  // listen for request on Blog posts
  emiter.on('fetching scType WpPosts', (data, promises) => {
    promises.push(new Promise(function(resolve, reject) {
      let conf = {};
      if (data.scContent && data.scContent.arg) {
        conf = data.scContent.arg;
      }
      wpFunc.fetchData(wpViewConf.apiUrl, '', conf, (err, val, headers) => {
        if (val) {
          data.scContent = Object.assign(
            data.scContent,
            {posts: val, headers: headers, conf: conf});
        }
        resolve();
      });
    }));
  });

  var promises = [];

  // get list of Blog Categories
  promises.push(new Promise(function(resolve) {
    wpFunc.fetchData(wpViewConf.apiUrl, 'categories', '', (err, val) => {
      if (val) {
        wpViewConf.categories = val;
      }
      resolve();
    });
  }));

  // setup object for data for theme
  themeData.wpConfig = wpViewConf;

  // done here
  Promise.all(promises).then(() => next());
}
module.exports = middleware;
