/**
 * page for displaying WP blog
 */

import React from 'react';
import { Grid, Row, Col, ButtonGroup, Button } from 'react-bootstrap';
import { connect } from 'react-redux';
import { LinkContainer } from 'react-router-bootstrap';
import { IndexLink } from 'react-router';
import { getComponent } from '../../../../themes/zealder-default-theme/src/routes/';
import {fetchData} from '../helpers/functions';
var _ = require('lodash');

class PgBlogContainer extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      scContent: {},
      activePage: (props.params.pageNum)?props.params.pageNum:1,
      isFetching: false
    };

    if (props.params.catSlug) {
      // when browsing specific category
      let cat = _.find(props.wpConfig.categories, ['slug', props.params.catSlug]);
      if (cat) {
        this.state.categories = cat['id'];
      }
      else this.state.categories = 'mustbewrongid';
    }

    // init content
    if (props.contentList) {
      let val = _.find(props.contentList, ['scId', 'wp-posts-index']);
      // all conditions that must be met to use this content
      if (_.isEmpty(val)) {}
      else if (!val.scContent.conf.page && 1 !== this.state.activePage) {}
      else if (val.scContent.conf.page &&
        val.scContent.conf.page !== this.state.activePage) {}
      else if (this.state.categories !== val.scContent.conf.categories) {}
      else {
        this.state.scContent = val.scContent;
      }
    }

    this.getData = this.getData.bind(this);
  }

  getData() {
    const {wpConfig} = this.props;

    this.setState({isFetching: true});

    let arg = {page: this.state.activePage};

    if (this.state.categories) {
      arg.categories = this.state.categories;
    }

    fetchData(wpConfig.apiUrl, 'posts', arg, (err, val, headers) => {
      if (val) {
        this.setState({isFetching: false, scContent: Object.assign(
          this.state.scContent,
          {posts:val, headers: headers}
        )});
      }
    });
  }

  componentWillReceiveProps(nextProps) {
    const {params, wpConfig} = this.props;
    // change state when pagination updates
    if (nextProps.params.pageNum &&
      nextProps.params.pageNum !== this.state.activePage) {
      this.setState({activePage: nextProps.params.pageNum}, this.getData);
    }
    else if (nextProps.params.catSlug !== params.catSlug) {
      // changed category
      let cat = _.find(wpConfig.categories, ['slug', nextProps.params.catSlug]);
      if (cat) {
        this.setState({categories: cat['id']}, this.getData);
      }
      else this.setState({categories: 'mustbewrongid'}, this.getData);
    }
  }

  // TODO: make display Loader when fecthing posts
  render() {
    const {wpConfig, blogPage} = this.props;
    const {scContent, activePage, isFetching} = this.state;
    let PgStandard = getComponent('PgStandard');
    let WpPosts = getComponent('WpPosts');

    let bodyNodes;
    if (isFetching) {
      // display loader while fetching
      let Loader = getComponent('Loader', '');
      bodyNodes = <Loader />;
    }
    else {
      // determine pager links
      let pagination, nextPath, prevPath;
      if (scContent.headers && 1 < Number(scContent.headers['x-wp-totalpages'])) {
        let totalpages = Number(scContent.headers['x-wp-totalpages']);
        if (activePage < totalpages) {
          nextPath = `${blogPage.realPath}/page/`+ (Number(activePage) + 1);
        }
        if (2 === Number(activePage)) {
          prevPath = blogPage.realPath;
        }
        else if (activePage > 1) {
          prevPath = `${blogPage.realPath}/page/`+ (Number(activePage) - 1);
        }

        pagination = (
          <div className="pagination-wrapper">
            <ButtonGroup bsSize="large">
              {prevPath &&
                <LinkContainer to={prevPath}><Button>PREVIOUS</Button></LinkContainer>
              }
              {nextPath &&
                <LinkContainer to={nextPath}><Button>NEXT</Button></LinkContainer>
              }
            </ButtonGroup>
          </div>
        )
      }

      bodyNodes = (
        <div>
          <WpPosts content={{scContent: scContent}}
            gridSize={{xs: 12, md:8, lg:9}} />
          {pagination}
        </div>
      );
    }

    return (
      <PgStandard {...this.props} bodyContent={
        <Grid fluid className="component-container">
          <Grid>
            <Row>
              <Col xs={12} md={8} lg={9}>
                {bodyNodes}
              </Col>
              <Col xs={12} md={4} lg={3}>
                <div className="side-bar">
                  <h4>BROWSE BY TOPIC</h4>
                  <ul className="categories">
                    {wpConfig.categories.map((cat, indx) => {
                      return (
                        <li key={indx}>
                          <IndexLink to={`${blogPage.realPath}/topics/${cat.slug}`}>
                            {cat.name}
                          </IndexLink>
                        </li>
                      );
                    })}
                  </ul>
                </div>
              </Col>
            </Row>
          </Grid>
        </Grid>
      } />
    )
  }

  componentDidMount() {
    const {wpConfig} = this.props;
    // if no content, let's get some
    if (_.isEmpty(this.state.scContent) && wpConfig && wpConfig.apiUrl) {
      this.getData();
    }
  }
}

const PgBlog = connect(
  (state) => {
  return {
    wpConfig: state.themeData.wpConfig,
    blogPage: _.find(state.pages, function(o) {
      return o.scsOptions && 'blog' === o.scsOptions.UniId;
    })
  }
}
)(PgBlogContainer)
module.exports = PgBlog;
