/**
 * page for displaying WP blog
 */

import React from 'react';
import { Grid, Row, Col, ResponsiveEmbed} from 'react-bootstrap';
import { connect } from 'react-redux';
import { IndexLink } from 'react-router';
import { getComponent } from '../../../../themes/zealder-default-theme/src/routes/';
import {fetchData} from '../helpers/functions';
var _ = require('lodash');
var moment = require('moment');

class PgBlogPostContainer extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      scContent: {},
      isFetching: false
    };

    // init content
    if (props.contentList) {
      let val = _.find(props.contentList, ['scId', `post-${props.params.slug}`]);
      // all conditions that must be met to use this content
      if (!_.isEmpty(val)) {
        this.state.scContent = val.scContent;
      }
    }

    this.getData = this.getData.bind(this);
  }

  getData() {
    const {wpConfig, params} = this.props;

    this.setState({isFetching: true});

    fetchData(wpConfig.apiUrl, 'post', {slug: params.slug}, (err, val) => {
      if (val && val[0]) {
        this.setState({isFetching: false, scContent: val[0]});
      }
    });
  }

  render() {
    const {wpConfig, blogPage} = this.props;
    const {scContent, isFetching} = this.state;
    let PgStandard = getComponent('PgStandard');

    let post;
    if (isFetching) {
      // display loader while fetching
      let Loader = getComponent('Loader', '');
      post = <Loader />;
    }
    else if (!_.isEmpty(scContent)) {
      // prep terms
      var categ;
      if (scContent.categories && wpConfig && wpConfig.categories) {
        categ = scContent.categories.map((catId, indx) => {
          let cat = _.find(wpConfig.categories, ['id', catId]);
          if (cat && cat.name) {
            return <span key={indx}>{0 !== indx ? ', ': ''}{cat.name}</span>
          }
          return null;
        });
      }

      // video or photo
      let featuredMedia;
      if (scContent.featured_video_url) {
        let vidUrl = scContent.featured_video_url;
        if (vidUrl.includes('www.youtube.com')) {
          // for youtube
          let yt = /embed\/([^/?]+)/.exec(vidUrl);
          if (yt && yt[1]) {
            vidUrl = `//www.youtube.com/embed/${yt[1]}?rel=0`;
          }
        }
        featuredMedia = (
          <ResponsiveEmbed a16by9>
            <iframe title="video" src={vidUrl} />
          </ResponsiveEmbed>
        );
      }
      else if (scContent.featured_mediaData) {
        featuredMedia = (
          <img
            src={scContent.featured_mediaData.source_url}
            alt={scContent.featured_mediaData.alt_text} />
        );
      }

      post = (
        <div className="blog-post">
          <div className="post-attrib">
            <span>
              Published: {moment(scContent.date).format("MMMM DD, YYYY")}
            </span>
            {categ &&
              <span>Topics: {categ}</span>
            }
          </div>
          <h1 dangerouslySetInnerHTML={{__html: scContent.title.rendered}} />
          <div className="post-photo">
            {featuredMedia}
          </div>
          <div dangerouslySetInnerHTML={{__html: scContent.content.rendered}} />
        </div>
      );
    }

    return (
      <PgStandard {...this.props} bodyContent={
        <Grid fluid className="component-container">
          <Grid>
            <Row>
              <Col xs={12} md={8} lg={8}>
                {post}
              </Col>
              <Col xs={12} md={4} lg={3} lgOffset={1}>
                <div className="side-bar">
                  <h4>BROWSE BY TOPIC</h4>
                  <ul className="categories">
                    {wpConfig.categories.map((cat, indx) => {
                      return (
                        <li key={indx}>
                          <IndexLink to={`${blogPage.realPath}/topics/${cat.slug}`}>
                            {cat.name}
                          </IndexLink>
                        </li>
                      );
                    })}
                  </ul>
                </div>
              </Col>
            </Row>
          </Grid>
        </Grid>
      } />
    )
  }

  componentDidMount() {
    const {wpConfig} = this.props;
    // if no content, let's get some
    if (_.isEmpty(this.state.scContent) && wpConfig && wpConfig.apiUrl) {
      this.getData();
    }
  }
}

const PgBlogPost = connect(
  (state) => {
  return {
    wpConfig: state.themeData.wpConfig,
    blogPage: _.find(state.pages, function(o) {
      return o.scsOptions && 'blog' === o.scsOptions.UniId;
    })
  }
}
)(PgBlogPostContainer)
module.exports = PgBlogPost;
