// @flow
'use strict';

var path = require('path');
var request = require('request');
var _ = require('lodash');

function render(req, res) {
  const { zlSite, themeData } = res.locals;

  let managingSite = themeData.authSites[themeData.managingSite];

  // send request to get a copy of the page that they're editing
  let urlEnd = req.originalUrl;
  if (zlSite.zsBaseUrlPath) {
    urlEnd = urlEnd.replace(`/${zlSite.zsBaseUrlPath}`, '');
  }
  if (managingSite.zsBaseUrlPath) {
    // add zsBaseUrlPath
    urlEnd = `/${managingSite.zsBaseUrlPath}${urlEnd}`;
  }
  var options = {
    // uses local url in case their domain is offline. But use their site path
    // TODO: may have to receive managingSite and path/page from GET params 
    url: `${req.protocol}://${req.get('host')}${urlEnd}`,
    headers: {
      'User-Agent': 'zealder-builder',
      'X-Managing-Site': themeData.managingSite,
      'X-Access-Token': req.accessToken.id
    }
  };
  req.app.debug('Requesting page for Builder: %s', options.url);
  request(options, function (error, response, body) {
    if (error) req.app.error(error);

    let appHtml = '', zlSiteJson = {};

    // extract app content
    var re = /<!--zealder-app:start--><div id="app-container">(.+)<\/div><!--zealder-app:end-->/;
    var content = body.match(re);
    if (content && content[1]) {
      // assign page content
      appHtml = content[1];
    }

    // extract app stylesheets
    var re = /href="([^\"]+app[^\"]+css)"/;
    var css = body.match(re);
    if (css && css[1]) {
      let cssUrl = css[1];
      if (_.startsWith(cssUrl, '//')) {
        cssUrl = `${req.protocol}:${cssUrl}`;
      }
      else if (_.startsWith(cssUrl, '/')) {
        cssUrl = `${req.protocol}:${req.get('host')}${cssUrl}`;
      }
      zlSiteJson.config = {
        canvas: {
          styles: [cssUrl],
          scripts: ['/assets/zealder-builder/js/canvas.js']
        }
      }
    }

    // render index
    res.locals.appHtml = appHtml;
    res.locals.zlSiteJson = JSON.stringify(zlSiteJson);
    res.render('index');
  });
}
module.exports = render;
