import React from 'react'
// we'll use this to render our app to an html string
import { renderToString } from 'react-dom/server'
// and these to match the url to routes and then render
import { match, RouterContext } from 'react-router'
import routes from './src/routes'
import { Provider } from 'react-redux'
import { createStore, applyMiddleware, compose } from 'redux'
import reducers from './src/redux/reducers';
import { selectPage, receiveContents } from './src/redux/actions/pages';
import thunk from 'redux-thunk';
var _ = require('lodash');

const middlewares = [thunk];

function render(req, res) {
  // values to make available to public js
  let zlSite = res.locals.zlSite;
  var zlSiteJson = {
    zlSite: {
      zsId: zlSite.zsId,
      zsName: zlSite.zsName,
      baseUrlPath: (zlSite.zsBaseUrlPath) ? zlSite.zsBaseUrlPath : '',
      zsTheme: zlSite.zsTheme,
      logo: (zlSite.zsOptions) ? zlSite.zsOptions.logo : '',
      homePage: (zlSite.zsOptions) ? zlSite.zsOptions.homePage : '',
      zsDefaultContent: zlSite.zsDefaultContent,
      accessToken: req.accessToken ? req.accessToken.id:'',
      user: zlSite.user ? zlSite.user : '',
      gtags: (zlSite.zsOptions) ? zlSite.zsOptions.gtags : '',
      realm: (zlSite.zsOptions && zlSite.zsOptions.realm) ? zlSite.zsOptions.realm: zlSite.zsId,
      variantActive: 'default'
    },
    pages: zlSite.pages,
    themeData: res.locals.themeData,
    siteContents: zlSite.siteContents
  };

  let store = compose(applyMiddleware(...middlewares))(createStore)(reducers, zlSiteJson);

  // match the routes to the url
  match(
    { routes: routes(zlSiteJson, req, res), location: req.url },
    (err, redirect, props) => {
    if (err) {
      // there was an error somewhere during route matching
      res.status(500).send(err.message)
    } else if (redirect) {
      // handle redirects
      res.redirect(redirect.pathname + redirect.search)
    } else if (props) {
      // if we got props then we matched a route and can render

      // get init/current page
      if (props.components[2] && props.components[2].scsId) {
        let scsId = props.components[2].scsId;
        let initPage = _.find(zlSite.pages, ['scsId', scsId]);
        req.app.debug('init page:%s', scsId);
        // load meta
        if (initPage) {
          // meta title
          if (initPage.scsHtmlMeta && initPage.scsHtmlMeta.title) {
            res.locals.title = initPage.scsHtmlMeta.title;
          }
          else if (initPage.scsName) {
            res.locals.title = `${initPage.scsName} : ${zlSite.zsName}`;
          }
          // meta robots
          if (initPage.scsHtmlMeta && initPage.scsHtmlMeta.noindex) {
            res.locals.robots = 'NOINDEX, NOFOLLOW';
          }
          // description
          if (initPage.scsHtmlMeta && initPage.scsHtmlMeta.description) {
            res.locals.description = initPage.scsHtmlMeta.description;
          }
          // keywords
          if (initPage.scsHtmlMeta && initPage.scsHtmlMeta.keywords) {
            res.locals.keywords = initPage.scsHtmlMeta.keywords;
          }
        }
        // get init/current page contents
        req.app.models.zlSiteContentIn.contentsToDisplay(
          req, res, scsId, null, (err, data) => {
            if (err) req.app.error(err);

            let contents = [];
            if (!_.isEmpty(data)) {
              data.forEach(function(dat) {
                // data from loopback need converting
                if (dat.toJSON) contents.push(dat.toJSON());
                else contents.push(dat);
              });
            }
            else req.app.debug('No content found for init page');

            store.dispatch(selectPage(scsId));
            store.dispatch(receiveContents(scsId, {data: contents}));

            // render index template
            renderResult(res, store, props);
          });
      }
      else {
        req.app.error('couldnt load init page from server-side');
        // when we don't know init page (not sure why we wouldn't)
        renderResult(res, store, props);
      }
    } else {
      // no errors, no redirect, we just didn't match anything
      res.status(404).send('Not Found')
    }
  })
};
module.exports = render;

function renderResult(res, store, props) {
  res.locals.zlSiteJson = JSON.stringify(store.getState());

  // server side rendering of react components
  res.locals.appHtml = renderToString(
    <Provider store={store}><RouterContext {...props}/></Provider>
  )
  // render index template
  res.render('index');
}
