/* global YT */

import React from 'react';
import { Carousel } from 'react-bootstrap';
import { preloadImage } from '../helpers/functions';
import { getComponent } from '../routes/';

export default class Background extends React.Component {
  componentDidMount() {
    const { background } = this.props;

    // for youtube background video
    if ('video' === background.type) {
      // load data
      var self = this, videoId = background.videoId;
      window.headerVideoDuration = false;
      window.headerVideoQualitySet = false;
      window.headerVideoFadedIn = false;

      // show video when in desktop size
      // get size of header
      var videoWidth = $( '.app-header' ).width();
      var videoHeight = videoWidth * 0.5625;
      if (videoHeight < $( '.app-header' ).height()) {
        videoHeight = $( '.app-header' ).height();
        videoWidth = videoHeight * 1.7777;
      }

      // loads the IFrame Player API code asynchronously.
      let scripts = Array
        .from(document.querySelectorAll('script'))
        .map(scr => scr.src);

      if (!scripts.includes('https://www.youtube.com/iframe_api')) {
        var tag = document.createElement('script');
        tag.src = "https://www.youtube.com/iframe_api";
        var firstScriptTag = document.getElementsByTagName('script')[0];
        firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);
      }
      else {
        // since already loaded, start player function
        window.onYouTubeIframeAPIReady();
      }

      // creates an <iframe> (and YouTube player)
      // after the API code downloads.
      window.onYouTubeIframeAPIReady = function() {
        new YT.Player('zeal-header-video', {
          height : videoHeight,
          width : videoWidth,
          videoId : videoId,
          playerVars: {
            controls: 0,
            rel: 0,
            showinfo: 0,
            modestbranding: 1,
            disablekb: 1,
            playsinline: 1,
          },
          events : {
            'onReady' : self.ytPlayerReady,
            'onStateChange' : self.ytPlayerStateChange
          }
        });
      }
    }
  }

  ytPlayerReady(event) {
		event.target.mute();
		event.target.playVideo();
	}

  ytPlayerStateChange(event) {
    if (!window.headerVideoQualitySet) {
      // get available qualities
      let qLevels = event.target.getAvailableQualityLevels();
      if (qLevels.length !== 0) {
        window.headerVideoQualitySet = true;
        if (event.target.getPlaybackQuality() !== qLevels[0]) {
          // use highest quality available
          event.target.setPlaybackQuality(qLevels[0]);
        }
      }
    }

    if (event.data == YT.PlayerState.PLAYING && !window.headerVideoFadedIn) {
      // fadeout the alt image (delay a few sec so yt logo hides first)
      window.headerVideoFadedIn = true;
      let fadeWaitLoop = setInterval(function() {
        if (3 < event.target.getCurrentTime()) {
          clearInterval(fadeWaitLoop);
          $('.app-header .component-background .video-background .image-component')
            .fadeOut(4000, 'swing');
        }
			}, 1000);
    }

    // make video loop
    var fadeTime = .1;
		if (event.data == YT.PlayerState.PLAYING && !window.headerVideoDuration) {
			window.headerVideoDuration = event.target.getDuration();
			let headerVideoStartFade = window.headerVideoDuration - fadeTime;

			window.headerVideoLooper = setInterval(function() {
				if (headerVideoStartFade <= event.target.getCurrentTime()) {
          //NOTE: Mac Safari still shows a slight delay
          event.target.seekTo(0, true);
				}
			}, 50);
    }
	}

  componentWillUnmount() {
    if (window.headerVideoLooper) {
      clearInterval(window.headerVideoLooper);
    }
  }

  render() {
    const { background, height, isHeader } = this.props;

    let Image = getComponent('Image', '');

    let className = 'component-background', style = {}, isParallax;
    /* to qualify for parallax it must supply height for Header since header
    parallax uses fixed div which need a height */
    if (background.parallax && (!isHeader || height)) {
      className += ' parallax'; isParallax = 1;
      if (height) {style.minHeight = height;}
    }

    // determine our background
    let bg;
    if (isParallax && 'image' === background.type) {
      // parallax image (no support for responsive images)
      bg = (
        <div style={{backgroundImage: `url('${background.src}')`}}
          className="image-component" />
      )
    }
    else if ('image' === background.type) {
      bg = <Image content={{scContent: background}} />;
    }
    else if ('carousel' === background.type) {
      // for carousel background
      const slides = background.scenes.map((item) => {
        // preload images
        preloadImage(item.src);

        return (
          <Carousel.Item key={item.src}>
            <Image content={{scContent: item}} />
          </Carousel.Item>
        );
      });
      bg = (
        <Carousel controls={false} indicators={false} pauseOnHover={false}>
          {slides}
        </Carousel>
      );
    }
    else if ('video' === background.type) {
      // for video background
      bg = (
        <div className="video-background">
          <div id="zeal-header-video" className="ytVideo" />
          <Image content={{scContent: background}} />
        </div>
      );
    }

    return (
      <div className={className} style={style}>
        {bg}
        <div className="backgroundTint"></div>
      </div>
    );
  }
}
