/**
 * For running Experiments like A/B tests and Multivariate tests from
 * Google Optimize
 *
 * to use:
 * - create Optimize experiment
 * - must have Targeting: "Evaluate on custom event: optimize.activate"
 * - URL Targeting should not allow 2 different experimemnts to target same page
 *   or it will produce strange results like page not update.
 *   should use Path Equal targetting to target specific page
 * - make variant with this Script Change on openning "body" tag:
 *   `if (window.setExpVariant){window.setExpVariant('scVariantId');}`
 */
import React from 'react';
import _ from 'lodash/core';
import { connect } from 'react-redux';
import { getComponent } from '../routes/';

class ExperimentContainer extends React.Component {
  render() {
    const { scContent } = this.props.content;

    // get children (actual component that will render)
    if (scContent && scContent.children) {
      // get change for variant if exist
      let variant = _.find(scContent.children, (o) => {
        return this.props.variantActive === o.scVariantId;
      });
      if (!variant) {
        // try default since there is no change in this variant
        variant = _.find(scContent.children, function(o) {
          return 'default' === o.scVariantId;
        });
      }
      if (!variant) {
        // not sure why we wouldn't have default, use first one
        variant = scContent.children[0];
      }
      if (variant) {
        let NodeComponent = getComponent(variant.scType, 'Html');
        return <NodeComponent content={variant} />
      }
    }

    // if we made it here return nothing
    return null;
  }
}

const Experiment = connect(
  (state) => {
  return {
    variantActive: state.zlSite.variantActive
  }
}
)(ExperimentContainer)

module.exports = Experiment;
