// @flow

import React from 'react';
import { Row, Col, Grid, ButtonGroup, Button } from 'react-bootstrap';
var _ = require('lodash');
var events = require('../../../../../socket/actions');
import { getComponent } from '../routes/';
import { connect } from 'react-redux';

class FormContainer extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      sc: props.content.scContent,
      formData: {},
      isLoading: false
    };
    this.populateFormData = this.populateFormData.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
  }

  handleSubmit(data) {
    if (!_.isEmpty(data) && this.state.sc.apiPost) {
      const self = this;
      this.setState({ isLoading: true, hasError: false });
      // send ajax update to server
      $.post(this.state.sc.apiPost, data)
        .always(() => {
          this.setState({ isLoading: false });
        })
       .done((response) => {
         // track event
         if ('gtag' in window) {
           let evAction = 'submit';
           if (this.state.sc.trackEventAction) {
             evAction = this.state.sc.trackEventAction;
           }
           let evParam;
           if (this.state.sc.trackEventParam) {
             evParam = this.state.sc.trackEventParam;
           }
           gtag('event', evAction, evParam);
         }

         // show confirmation
         let updFor = 'Data';
         // if update was for a model data, broadcast to pages of that model
         if (this.props.page && this.props.page.scsOptions.hasDataOf) {
           updFor = this.props.page.scsOptions.hasDataOf;
           socket.emit(
             events.EDIT_DATA_OF,
             this.props.page.scsOptions.hasDataOf);
         }
         // update data
         if (response.data) {
           this.setState({ formData: response.data });
         }

         // display confirmation
         if (response.data && response.data.responseHtml) {
           // do nothing because responseHtml will be confirmation
         }
         else if (response.message) {
           Messenger().success({message: `${response.message}`, id: 'save'});
         }
         else {
           Messenger().success({message: `${updFor} has been saved!`, id: 'save'});
         }

         // run any other functions set for component
         if (this.state.sc.doAfterSave) {
           if (this.state.sc.doAfterSave.eval) {
             eval(this.state.sc.doAfterSave.eval);
           }
         }
         // when child of TabContainer (multi-tab forms)
         if (this.props.parentTabs) {
           // share result with other tabs
           this.props.parentTabs.shareData(response.data);
           // jump to next Tab
           this.props.parentTabs.handleSelect(this.props.parentTabs.state.key + 1);
         }
       })
       .fail(err => {
         Messenger().error({message: window.defaultErrorMsg});
       });
    }
  }

  componentWillReceiveProps(props) {
    // if child of TabContainer (multi-tab forms)
    if (props.parentTabs && props.parentTabs.state.sharedData) {
      // update state with shared data
      this.populateFormData(props.parentTabs.state.sharedData);
    }
  }

  componentWillUpdate(props, state) {
    const {sc, formData} = state;
    if (sc.doWhenUpdating) eval(sc.doWhenUpdating);
  }

  populateFormData(formData) {
    // populate state formData with provided formData using only expected values
    let data = Object.assign({}, this.state.formData);
    _.forEach(this.state.sc.fields, function(field) {
      if (field.embedIn) {
        // for embedIn, the value will be in embed object (init load from model)
        let embed = formData[field.embedIn];
        if (embed && !_.isUndefined(embed[field.name])) {
          data[field.name] = formData[field.embedIn][field.name];
        }
      }
      else if (!_.isUndefined(formData[field.name])) {
        data[field.name] = formData[field.name];
      }
    });
    if (JSON.stringify(data) !== JSON.stringify(this.state.formData)) {
      this.setState({formData: data});
    }
  }

  componentDidMount() {
    const { content, zlSite, params } = this.props;

    if (params && params.id) {
      const id = params.id;
      // check if data was passed in location state (likely from a datatable)
      if (this.props.location.state && this.props.location.state.formData) {
        this.populateFormData(this.props.location.state.formData);
      }
      else if (this.state.sc.api) {
        // most likely landing page or not requested from a datatable
        // get existing data from api call
        $.get( `${this.state.sc.api}/${id}`)
         .done(data => this.populateFormData(data))
         // if fail for any reason, send to Add new
         .fail(() => {
           // display error
           Messenger().error({
             message: `This item you were trying to access no longer exist`,
             id: `missing${id}`});
           this.props.router.push({pathname: `/${zlSite.baseUrlPath}`})
         });
      }
    }
    else if (_.isEmpty(this.state.formData)) {
      // for new, assign default vals
      let formData = {};
      _.forEach(this.state.sc.fields, field => {
        // TODO: should also work for embeds
        formData[field.name] = field.value
      });
      if (!_.isEmpty(formData)) this.populateFormData(formData);
    }
  }

  render() {
    const { content } = this.props;
    const { sc, formData, isLoading } = this.state;

    if (formData && formData.responseHtml) {
      // since we received a responseHtml from a submit, display it
      return (
        <div dangerouslySetInnerHTML={{__html: formData.responseHtml}} />
      )
    }

    const FieldRenderer = props => (
      <Col xs={12} sm={(props.field.columns)?props.field.columns:6}>
        {props.children}
      </Col>
    );

    let FormBuilder = getComponent('FormBuilder', '');

    let btnRow, btnId = `save-btn-${content.scId}`;
    if ('body' !== content.scPlacement) {
      let buttonLabel = 'Save';
      if (content.scContent.buttonLabel) {
        buttonLabel = content.scContent.buttonLabel;
      }
      btnRow = (
        <Row>
          <Col xs={12} style={{textAlign: "right", marginTop: "2rem"}}>
            <ButtonGroup>
              <Button id={btnId} bsStyle="primary"
                bsSize="large" disabled={isLoading}>{buttonLabel}</Button>
            </ButtonGroup>
          </Col>
        </Row>
      );

    }

    let formAction;
    if (content.scContent.forModel) { formAction = content.scContent.forModel; }

    return (
      <div className="form-container">
        <Row>
          <FormBuilder fields={sc.fields}
            submitButton={`#${btnId}`} values={formData} formAction={formAction}
            handleSubmit={this.handleSubmit} FieldRenderer={FieldRenderer} />
        </Row>
        {btnRow}
      </div>
    );
  }
}

const Form = connect()(FormContainer)

module.exports = Form;
