// @flow

import React from 'react';
import { Modal } from 'react-bootstrap';
import { getComponent } from '../routes/';

export default class GlobalModal extends React.Component {
  constructor(props) {
    super(props);

    this.handleShow = this.handleShow.bind(this);
    this.handleClose = this.handleClose.bind(this);

    this.defaultState = {
      show: false,
      bsSize: null,
      title: '',
      subTitle: '',
      content: '',
      forceShow: false
    };
    this.state = this.defaultState;
  }

  handleClose() {
    // unpause PrefetchPage
    var index = window.pausePrefetchPageFor.indexOf('GlobalModal');
    if (index > -1) window.pausePrefetchPageFor.splice(index, 1);
    // update state
    this.setState(this.defaultState);
  }

  handleShow(props) {
    // let's pause PrefetchPage so form fields don't loose focus
    if (window.pausePrefetchPageFor) {
      window.pausePrefetchPageFor.push('GlobalModal');
    }

    // update state
    this.setState(Object.assign({ show: true }, props));
  }

  componentDidMount() {
    const { query } = this.props;

    // provide global function that any page can call to load modal
    window.loadAndShowModal = this.handleShow;
    window.hideGlobalModal = this.handleClose;

    // when a page query requested to display an initial modal
    if (query && query.initModal) {
      switch (query.initModal) {
        case 'reset-password':
          let RegisterOrLogin = getComponent('RegisterOrLogin', '');
          loadAndShowModal({
            title: 'Create new Password',
            content: <RegisterOrLogin altState={{currentScreen: 'resetPassword'}} />
          });
          break;
      }
    }
  }

  render() {
    return (
      <Modal show={this.state.show} onHide={this.handleClose}
        bsSize={this.state.bsSize} backdrop={(this.state.forceShow)?'static': true}
        >
        {(this.state.title || this.state.subTitle) &&
          <Modal.Header closeButton={(this.state.forceShow)?false: true}>
            {this.state.title &&
              <Modal.Title>{this.state.title}</Modal.Title>
            }
            {this.state.subTitle &&
              <div>{this.state.subTitle}</div>
            }
          </Modal.Header>
        }
        <Modal.Body>
          {this.state.content}
        </Modal.Body>
      </Modal>
    );
  }
}
