// @flow

import React from 'react';
import { Row, Col } from 'react-bootstrap';
import { getComponent } from '../routes/';

export default class Grid extends React.Component {
  render() {
    const { scContent, verticalAlign } = this.props.content;

    // css class
    let className = this.props.content.className;

    // get children (columns)
    let children = [];
    if (scContent && scContent.children) {

      children = scContent.children.map((item, index) => {
        let NodeComponent = getComponent(item.scType, 'Html');
        // responsive breakpoints
        let xs = item.xs ? item.xs : 12;
        let sm = item.sm ? item.sm : xs;
        let md = item.md ? item.md : sm;
        let lg = item.lg ? item.lg : md;

        let colClassName = (item.className) ? `${item.className} `: '';
        if (verticalAlign) {
          // forces all columns to align vertically
          colClassName += 'col-vertical-align';
        }

        // content for column
        let colContent = [];
        if (item.scContent && item.scContent.children) {
          colContent = item.scContent.children.map((colItem, colIndex) => {
            let ColComponent = getComponent(colItem.scType, 'Html');
            return <ColComponent content={colItem} key={`col-cont-${colIndex}`} />
          });
        }

        return (
          <Col xs={xs} sm={sm} md={md} lg={lg} key={`col-${index}`}
            className={colClassName} {...item.props}>
            {colContent}
          </Col>
        );
      });
    }
    else return null;

    return (
      <Row className={className}>{children}</Row>
    );
  }
}
