// @flow

import React from 'react';
import { connect } from 'react-redux';
import _ from 'lodash/core';
import { Nav, NavItem, NavDropdown, MenuItem, Accordion, Panel, ListGroup,
  ListGroupItem } from 'react-bootstrap';
import { LinkContainer, IndexLinkContainer } from 'react-router-bootstrap';
var bowser = require('bowser');

class MainMenuContainer extends React.Component {
  constructor(props) {
    super(props);

    this._setupHeader = this._setupHeader.bind(this);
  }

  componentDidMount() {
    if (this.props.dontWrap) {
      // when dontWrap, we force nav to single row using selector that was passed
      this._setupHeader();

      // also when browser size/orientation changes
      if (bowser.mobile || bowser.tablet) {
        $( window ).on( "orientationchange", this._setupHeader);
      }
      else {
        $( window ).on( "resize", this._setupHeader);
      }
    }
  }

  componentWillUnmount() {
    $( window ).off( "orientationchange", this._setupHeader );
    $( window ).off( "resize", this._setupHeader );
  }

  render() {
    const { zlSite, pages, pullRight, stacked, accordion, bsStyle } = this.props;
    let navItems;

    if (accordion) {
      // when requested to use accordion for subs
      navItems = this._makeAccordionNav(pages, 0, []);
      return (
        <Accordion id="main-menu-burger">{navItems}</Accordion>
      );
    }
    else {
      navItems = this._makeNav(pages, 0, []);
      return (
        <Nav bsStyle={(bsStyle?bsStyle:'pills')} pullRight={pullRight}
          stacked={stacked} >
          {navItems}
        </Nav>
      );
    }
  }

  // for getting path to page
  _getPath(path) {
    let returnPath = '/';
    if (this.props.zlSite.baseUrlPath) {
      returnPath += this.props.zlSite.baseUrlPath + '/';
    }
    return `${returnPath}${path}`;
  }

  // for building nav at specific level
  _makeNav(all, level, navItems, inDropDown = false) {
    var self = this;
    _.forEach(_.filter(all, {'scsParent': level}), function(page) {
      // skip hidden pages
      if (page.scsHidden)  return;

      // get subs
      let subNav = [];
      if (_.find(all, {'scsParent': page.scsId})) {
        subNav = self._makeNav(all, page.scsId, subNav, true);
      }

      if (!inDropDown && !_.isEmpty(subNav)) {
        // top menu items with children
        navItems.push(
          <NavDropdown key={`ndd-${page.scsId}`} data-top-menu-label={page.scsName}
            title={
              <span>
                <span className="zl-menu-label-text">
                  {page.scsName}
                </span> <i className="icon icon-fontello-down-open"></i>
              </span>
            } id={`ndd-${page.scsId}`}>
            {subNav}
          </NavDropdown>
        );
      }
      else {
        // determine url
        let url = '#';
        if ('link' === page.scsBase && page.scsOptions) {
          // for http link pages
          url = page.scsOptions.Url;
        }

        // determine type of item
        let item;
        if (inDropDown) {
          item =
            <MenuItem key={`menu-${page.scsId}`} href={url}
              className={`pg-slug-${page.scsSlug}`} header={!_.isEmpty(subNav)} >
              {page.scsName}
            </MenuItem>;
        }
        else {
          // top menu with no children
          item =
            <NavItem key={`menu-${page.scsId}`} className={`pg-slug-${page.scsSlug}`}
              href={url}>
              {page.scsName}
            </NavItem>;
        }

        // now push item to menu
        navItems.push(self._wrapNavLink(page, item));

        if (!_.isEmpty(subNav)) {
          // push subs also
          navItems.push(...subNav);
        }
      }
    });

    if (!inDropDown) {
      // lastly, add also backup for menu items that don't fit
      navItems.push(
        <NavDropdown key="ndd-backup" className="menu-ndd-backup"
          title={
            <span>
              <i>more</i> <i className="icon-fontello-down-open"></i>
            </span>
          } id="ndd-backup">
        </NavDropdown>
      );
    }

    return navItems;
  }

  // for building Accordion nav at specific level
  _makeAccordionNav(all, level, navItems, inDropDown = false) {
    var self = this;
    _.forEach(_.filter(all, {'scsParent': level}), function(page) {
      // skip hidden pages
      if (page.scsHidden)  return;

      // get subs
      let subNav = [];
      if (_.find(all, {'scsParent': page.scsId})) {
        subNav = self._makeAccordionNav(all, page.scsId, subNav, true);
      }

      if (!inDropDown && !_.isEmpty(subNav)) {
        // top menu with sub-menu
        navItems.push(
          <Panel key={`ndd-${page.scsId}`}>
            <Panel.Heading>{page.scsName}</Panel.Heading>
            <ListGroup fill >
              {subNav}
            </ListGroup>
          </Panel>
        );
      }
      else {
        // determine url
        let url = '#';
        if ('link' === page.scsBase && page.scsOptions) {
          // for http link pages
          url = page.scsOptions.Url;
        }

        // determine type of item
        let item =
          <ListGroupItem key={`menu-${page.scsId}`} href={url}>
            {page.scsName}
          </ListGroupItem>;

        item = self._wrapNavLink(page, item);

        // now push item to menu
        if (!inDropDown && self.props.accordion) {
          // this is top item in accordion that has no subs
          navItems.push(
            <Panel key={`ndd-${page.scsId}`}>
              <ListGroup fill >
                {item}
              </ListGroup>
            </Panel>
          );
        }
        else {
          navItems.push(item);
        }
      }
    });

    return navItems;
  }

  _wrapNavLink(page, item) {
    // for local zealder pages we wrap in IndexLinkContainer
    if ('link' === page.scsBase && (!page.scsOptions
      || 'internal page' !== page.scsOptions.linkTarget)) {
      // ignore external links
      return item;
    }

    if (page.scsId === this.props.zlSite.homePage) {
      // home page link
      return (
        <IndexLinkContainer key={`link-${page.scsId}`} to={this._getPath('')}>
          {item}
        </IndexLinkContainer>);
    }
    else {
      let to = this._getPath(page.path);
      if ('link' === page.scsBase) {
        // for internal links
        to = page.scsOptions.Url;
      }
      return (
        <LinkContainer key={`link-${page.scsId}`} to={to}>
          {item}
        </LinkContainer>);
    }
  }

  // makes sure header is not wrapped
  // TODO: replace with `unWrapMenu` function in 'src/helpers/functions';
  _setupHeader() {
		var offset, hasWrap, ele = this.props.dontWrap;
		// first reset style
		$(ele).css({
			"height" : "",
			"overflowY" : ""
		});

		$(`${ele} > .nav > li`).css("display", "");

    // get first visible nav item's position
    if (offset = $(`${ele} > .nav > li:visible:first`).offset()) {
      let backupItem = $(`${ele} > .nav > li.menu-ndd-backup`);
      // empty any previous elements added
      backupItem.find(".dropdown-menu").empty();

      // get all other main nav items in reverse order
  		$($(`${ele} > .nav > li:visible`).get().reverse()).each(function(index) {
        // determine if menu wraps
        if (!hasWrap) {
          let curOffset = $(this).offset();
          if (curOffset.top != offset.top) {
            hasWrap = true;
            // display backup
            backupItem.css("display", "inline-block");
          }
          else {
            // since menu doesn't wrap we don't need to do anything
            return false;
          }
        }

        if (hasWrap) {
          // check if backup is still wrap
          let curOffset = $(backupItem).offset();
          if (curOffset.top != offset.top) {
            // copy it's children
            /* TODO: the items that are clone force page load and doesn't use
            router. Must find a way to fix this. May have to preload a copy of
            all items to backup and simply iterate the ones we're hiding and
            find a value match to compare and force display of same key in backup */
            let subNavItem = $( this ).find( ".dropdown-menu" );
            if (subNavItem) {
              subNavItem.children().clone().prependTo(backupItem.find(".dropdown-menu"));
              // place a header in menu
              let labelTxt = $( this ).find( ".zl-menu-label-text" ).html();
              $( backupItem ).find(".dropdown-menu").prepend(
                `<li action="push" role="heading" class="dropdown-header">${labelTxt}</li>`
              );
            }
            else {
              // since it has no children, clone self
              $( this ).clone().prependTo(backupItem.find(".dropdown-menu"));
            }
            // as long as backup is wrap we keep hiding items
            $(this).css("display", "none");
          }
          else {
            // we can stop iterating since backup is no longer wrap
            return false;
          }
        }
  		});
    }

		// now we can remove fixed height on mainNav
		$(ele).css({
			"height" : "auto",
			"overflowY" : "visible"
		});
	}
}

const MainMenu = connect(
  (state) => {
  return {
    pages: _.sortBy(state.pages, 'scsOrder'),
    zlSite: state.zlSite
  }
}
)(MainMenuContainer)

module.exports = MainMenu;
