// @flow

import React from 'react';
import { connect } from 'react-redux';
import _ from 'lodash/core';
import { Nav, NavItem, NavDropdown, MenuItem } from 'react-bootstrap';
import { LinkContainer } from 'react-router-bootstrap';
import { getComponent } from '../routes/';
import { updateThemeData } from '../redux/actions/theme';

class PersonBarContainer extends React.Component {
  componentWillUnmount() {
    /* save a copy of google_translate_element (if exist) to persist in
    subsequent updates since react may destroy the placeholder div */
    window.translatorEle = $( "#google_translate_element" );
  }

  componentDidMount() {
    // check if a google_translate_element existed before and persist it
    if ('translatorEle' in window) {
      $( "#google_translate_element" ).replaceWith(window.translatorEle);
    }

    // check if using currency converter
    if (this.props.currencySelected) {
      $('html').addClass('currency-selected');
    }
    else $('html').removeClass('currency-selected');
  }

  componentDidUpdate(prevProps, prevState) {
    // check if using currency converter
    if (this.props.currencySelected) {
      $('html').addClass('currency-selected');
    }
    else $('html').removeClass('currency-selected');
  }

  render() {
    const { SocialNetworks, pullRight, zsDefaultContent, user, memberPage,
      baseUrlPath, currencyList, currencySelected, dispatch, stacked } = this.props;

    let conf;
    if (zsDefaultContent && zsDefaultContent.personBar) {
      conf = zsDefaultContent.personBar;
    }

    // item for social icons
    let items = [];
    if (conf && conf.social && SocialNetworks) {
      _.forEach({Facebook: 'icon icon-fontello-facebook-rect-1'}, (val, key) => {
        if (SocialNetworks[key]) {
          items.push(
            <NavItem key={`sn-${key}`} href={SocialNetworks[key].url}>
              <i className={val}></i>
            </NavItem>
          )
        }
      })
    }

    // item for phone
    if (conf && conf.phone) {
      items.push(
        <NavItem key='pbar-phone' href={`tel:${conf.phone}`} className="phone-btn">
          <i className="icon icon-fontello-phone-1" /> <span className="number">
            {conf.phone}
          </span>
        </NavItem>
      )
    }

    if (user) {
      // member button
      if (memberPage && 'link' == memberPage.scsBase) {
        // use href/http
        items.push(
          <NavItem key='pbar-member' href={memberPage.scsOptions.Url} className="member-btn">
            <i className="icon icon-fontello-user-1"></i> My Account
          </NavItem>
        )
      }
      else if (memberPage) {
        let path = '';
        if (baseUrlPath) path += `${baseUrlPath}/`;
        path += memberPage.path;
        // use js route
        items.push(
          <LinkContainer key='pbar-member' to={memberPage.path} className="member-btn">
            <NavItem>
              <i className="icon icon-fontello-user-1"></i> My Account
            </NavItem>
          </LinkContainer>
        )
      }
    }
    else {
      // login button
      let label = 'Sign In', modalTitle = 'Sign in or register';
      if (conf && conf.login) {
        // props
        if (conf.login.label) label = conf.login.label;
        if (conf.login.modalTitle) modalTitle = conf.login.modalTitle;
      }
      let RegisterOrLogin = getComponent('RegisterOrLogin', '');
      items.push(
        <NavItem key='pbar-login' className="login-btn" onClick={() => {
          loadAndShowModal({
            title: modalTitle,
            content: <RegisterOrLogin altState={{currentScreen: 'login'}} />
          })
        }}>
          <i className="icon icon-fontello-login-1"></i> {label}
        </NavItem>
      )

      if (conf && conf.signup) {
        // register button
        items.push(
          <NavItem key='pbar-register' className="register-btn" onClick={() => {
            loadAndShowModal({
              title: 'Sign in or register',
              content: <RegisterOrLogin altState={{currentScreen: 'register'}} />
            })
          }}>
            <i className="icon icon-fontello-user-add-1"></i> {conf.signup.label}
          </NavItem>
        )
      }
    }

    if (!conf || !conf.noContact) {
      // contact button
      items.push(
        <LinkContainer key='pbar-contact' to='/Contact'>
          <NavItem>
            <i className="icon icon-fontello-email"></i> Contact
          </NavItem>
        </LinkContainer>
      )
    }

    // item for currency Converter
    if (conf && conf.currencyConverter && currencyList) {
      items.push(
        <NavDropdown key="pbar-currency" className="pbar-currency"
          title={
            <span><i className="icon icon-fontello-calculator"></i> Currency</span>
          } id="pbar-currency"
          onSelect={(key, e) => {
            dispatch(updateThemeData('currencySelected', e.target.dataset.val));
          }}
        >
        <MenuItem key={`pbar-cur-opt`} data-val="" active={!currencySelected}>
          no currency conversion
        </MenuItem>
        {currencyList.map((cur) => (
          <MenuItem key={`pbar-cur-opt-${cur.curCode}`} data-val={cur.curCode}
            active={currencySelected === cur.curCode}>
            {cur.curFullName} ({cur.curCode})
          </MenuItem>
        ))}
        </NavDropdown>
      )
    }

    // item for translator
    if (conf && conf.translator) {
      items.push(
        <NavItem key='pbar-translator'>
          <div id="google_translate_element" />
        </NavItem>
      )
    }

    return (
      <Nav className="person-bar" bsStyle="pills" pullRight={pullRight} stacked={stacked}>
        {(items && items[0])? items: null}
      </Nav>
    )
  }
}

const PersonBar = connect(
  (state) => {
  return {
    zsDefaultContent: state.zlSite.zsDefaultContent,
    baseUrlPath: state.zlSite.baseUrlPath,
    user: state.zlSite.user,
    memberPage: _.find(state.pages, function(o) {
      return o.scsOptions && 'member-home' === o.scsOptions.UniId;
    }),
    SocialNetworks: state.themeData.SocialNetworks,
    currencyList: state.themeData.currencyList,
    currencySelected: state.themeData.currencySelected
  }
}
)(PersonBarContainer)

module.exports = PersonBar;
