/**
 * allows direct access to react components
 */
import _ from 'lodash/core';
import React from 'react';
import * as bootstrap from 'react-bootstrap';
import { getComponent } from '../routes/';
import { IndexLink } from 'react-router';

export default class ReactComponent extends React.Component {
  render() {
    if (this.props.content) {
      const {content, ...passProps} = this.props;
      const { scType, scContent } = content;
      let Component;
      // check if react-bootstrap component
      if (scType.startsWith("Bootstrap")) {
        let type = scType.replace('Bootstrap', '');
        Component = bootstrap[type];
      }
      else if (RegExp('^[a-z]').test(scType)) {
        if ('routerIndexLink' === scType) {
          Component = IndexLink;
        }
        else {
          // DOM tag components
          Component = scType;
        }
      }

      if (Component) {
        // props to pass to component including passProps from parent (some
        // parent that has listeners need child props to include props eg,
        // BootstrapOverlayTrigger)
        let props = Object.assign({}, passProps, scContent);

        if (!_.isEmpty(scContent)) {
          // check all props for sub components. Primarily only 'children' prop
          // will have this but other props may have also (ie
          // BootstrapPanel.header can be a React component)
          _.forEach(scContent, function(val, key) {
            // for children arrays
            if ('children' === key && Array.isArray(val)) {
              props.children = val.map((item, index) => {
                if (item && item.scType) {
                  let SubComponent = getComponent(item.scType, '');
                  return <SubComponent content={item} key={`comp-${index}`} />
                }
                return null;
              });
            }
            else if (!_.isEmpty(val) && val.scType) {
              let SubComponent = getComponent(val.scType, '');
              props[key] = <SubComponent content={val} />;
            }
          });
        }

        return <Component {...props} />
      }
    }

    // worst case scenerio
    return null;
  }
}
