// @flow

import React from 'react';
import { Alert, Button, Row, Col } from 'react-bootstrap';
import _ from 'lodash/core';
import { getComponent } from '../routes/';
import { connect } from 'react-redux';
import { updateUserToken } from '../redux/actions/zlSite';

class RegisterOrLoginContainer extends React.Component {
  constructor(props) {
    super(props);

    let defaultState = {
      currentScreen: 'register',
      modeConf: {
        register: { btnLabel: 'Register', btnLabelLoad: 'Registering...' },
        login: { btnLabel: 'Sign in', btnLabelLoad: 'Signing in...' },
        forgot: { btnLabel: 'Reset Password', btnLabelLoad: 'Reseting Password...' },
        resetPassword: { btnLabel: 'Reset Password', btnLabelLoad: 'Updating Password...' }
      },
      isLoading: false,
      hasError: false
    };
    this.state = Object.assign({}, defaultState, props.altState);

    if (props.user && props.user.usrId && 'resetPassword' !== this.state.currentScreen) {
      // if already signed in, do postLogin
      this.postLogin();
    }
    else if ('resetPassword' === this.state.currentScreen && !props.user) {
      // not signed in; cant change pwd (maybe reset token expired)
      this.state = Object.assign(this.state, {
        currentScreen: 'forgot',
        hasError: 'The reset token has expired. You must restart process.'
      });
    }

    this.handleSubmit = this.handleSubmit.bind(this);
    this.postLogin = this.postLogin.bind(this);
  }

  handleSubmit(data, cb) {
    this.setState({ isLoading: true, hasError: false });

    const { currentScreen } = this.state;
    let apiUrl;

    if ('login' === currentScreen) apiUrl = '/api/zlUsers/login';
    else if ('register' === currentScreen) apiUrl = '/api/zlUsers';
    else if ('forgot' === currentScreen) apiUrl = '/api/zlUsers/reset';
    else if ('resetPassword' === currentScreen) apiUrl = '/api/zlUsers/reset-password';

    $.post( apiUrl, data)
      .always(() => {
        this.setState({ isLoading: false });
      })
     .done(response => {
       if ('forgot' === currentScreen) {
         this.setState({ doneNotice:
           'An email with a reset password link has been sent to your email '+
           'address. Please check your email for further instructions.'
         });
       }
       else if ('resetPassword' === currentScreen) {
         this.setState({ currentScreen: 'login' });
       }
       else if (response && response.id) this.postLogin(response);
       else this.setState({ hasError: window.defaultErrorMsg });
     })
     .fail((err) => {
       if (err.responseJSON && err.responseJSON.error) {
         if ('ER_DUP_ENTRY' === err.responseJSON.error.code) {
           this.setState({ hasError: 'An account with this email address already exist.' });
         }
         else this.setState({ hasError: err.responseJSON.error.message });
       }
     });
  }

  postLogin(response) {
    const { currentScreen } = this.state;

    if (response && response.id) {
      // persist user/token to store
      this.props.dispatch(updateUserToken(response));
    }
    // hide GlobalModal (if displayed)
    window.hideGlobalModal();

    // track event
    if ('gtag' in window) {
      let action = currentScreen;
      if ('register' === currentScreen) {
        action = 'sign_up';
      }
      gtag('event', action);
    }

    if (this.props.doAfterLogin) {
      this.props.doAfterLogin();
    }
    else if (this.props.memberPage) {
      // go to member-home page
      if ('link' === this.props.memberPage.scsBase) {
        let url = this.props.memberPage.scsOptions.Url;
        // pass along token
        if (url.includes('?')) url += '&';
        else url += '?';
        if (response) url += `access_token=${response.id}`;
        window.location.href = url;
        Messenger().info({
          message: 'Signing in. Please wait...',
          id: 'SigninIn',
          hideAfter: 0
        });
      }
      else window.routeToPage(window.uniPages['member-home']);
    }
  }

  render() {
    let FormBuilder = getComponent('FormBuilder', '');
    let btnId = 'registerOrLoginBtn';

    const FieldRenderer = props => (
      <Col xs={12} sm={6}>{props.children}</Col>
    );

    const { isLoading, currentScreen, modeConf, hasError, doneNotice } = this.state;

    // login notice
    let loginNoticeEle;
    try {
      // test if theme provides own login notice
      let LoginInfo = getComponent('LoginInfo', '');
      loginNoticeEle = <LoginInfo />
    }
    catch (err) {
      // default login notice
      loginNoticeEle = (
        <Alert bsStyle="warning">
          Not a member? <a href="javascript:void(0)"
            onClick={() => this.setState({ currentScreen: 'register' })}>
            Register for free
          </a>
        </Alert>
      )
    }

    if (doneNotice) {
      return (
        <div>
          <Alert bsStyle="warning">{doneNotice}</Alert>
        </div>
      )
    }

    return (
      <div>
        {hasError &&
          <Alert bsStyle="danger">
            An error occurred: {hasError}
          </Alert>
        }
        {'login' === currentScreen &&
          <div>
            {loginNoticeEle}
            <Row>
              <FormBuilder fields={fields.login} submitButton={`#${btnId}`}
                handleSubmit={this.handleSubmit} autoFocus={false}
                FieldRenderer={FieldRenderer} formAction="login" />
            </Row>
            <Row>
              <Col xsHidden xs={6}>
              </Col>
              <Col xs={12} xs={6}>
                <a href="javascript:void(0)"
                  onClick={() => this.setState({ currentScreen: 'forgot' })}>
                  Forgot Password?
                </a>
              </Col>
            </Row>
          </div>
        }
        {'register' === currentScreen &&
          <div>
            <Alert bsStyle="warning">
              Already a member? <a href="javascript:void(0)"
                onClick={() => this.setState({ currentScreen: 'login' })}>
                Sign in
              </a>
            </Alert>
            <Row>
              <FormBuilder fields={fields.register} submitButton={`#${btnId}`}
                handleSubmit={this.handleSubmit} autoFocus={false}
                FieldRenderer={FieldRenderer} formAction="signup" />
            </Row>
          </div>
        }
        {'forgot' === currentScreen &&
          <div>
            <Alert bsStyle="warning">
              <div>Enter your email address and we will email you a new password.</div>
              <a href="javascript:void(0)"
                onClick={() => this.setState({ currentScreen: 'login' })}>
                Back to Sign in
              </a>
            </Alert>
            <FormBuilder fields={fields.forgot} submitButton={`#${btnId}`}
              handleSubmit={this.handleSubmit} autoFocus={false} />
          </div>
        }
        {'resetPassword' === currentScreen &&
          <div>
            <Alert bsStyle="warning">
              Enter your new password below to complete the password reset process
            </Alert>
            <Row>
              <FormBuilder fields={fields.resetPassword} submitButton={`#${btnId}`}
                handleSubmit={this.handleSubmit} autoFocus={false}
                FieldRenderer={FieldRenderer} />
            </Row>
          </div>
        }
        <div style={{textAlign: 'right'}}>
          <Button id={btnId} bsStyle="primary" type="button"
            disabled={isLoading}
            >
            {isLoading ? modeConf[currentScreen].btnLabelLoad : modeConf[currentScreen].btnLabel}
          </Button>
        </div>
      </div>
    )
  }
}

const RegisterOrLogin = connect(
  (state) => {
  return {
    user: state.zlSite.user,
    memberPage: _.find(state.pages, function(o) {
      return o.scsOptions && 'member-home' === o.scsOptions.UniId;
    })
  }
}
)(RegisterOrLoginContainer)
module.exports = RegisterOrLogin;

const fields = {
  register: [
    {
      name: 'usrFirstName',
      format: 'text',
      placeholder: 'First Name',
      validations: { required: true, allowEmpty: false, maxLength: 30 }
    },
    {
      name: 'usrLastName',
      format: 'text',
      placeholder: 'Last Name',
      validations: { required: true, allowEmpty: false, maxLength: 50 }
    },
    {
      name: 'email',
      format: 'email',
      placeholder: 'Email',
      validations: { required: true, allowEmpty: false, format: 'email', maxLength: 255 }
    },
    {
      name: 'password',
      format: 'password',
      placeholder: 'Password',
      validations: { required: true, allowEmpty: false, maxLength: 20, minLength: 6 }
    }
  ],
  login: [
    {
      name: 'email',
      format: 'email',
      placeholder: 'Email',
      validations: { required: true, allowEmpty: false, format: 'email', maxLength: 255 }
    },
    {
      name: 'password',
      format: 'password',
      placeholder: 'Password',
      validations: { required: true, allowEmpty: false }
    }
  ],
  forgot: [
    {
      name: 'email',
      format: 'email',
      placeholder: 'Email',
      validations: { required: true, allowEmpty: false, format: 'email', maxLength: 255 }
    }
  ],
  resetPassword: [
    {
      name: 'newPassword',
      format: 'password',
      placeholder: 'New Password',
      validations: { required: true, allowEmpty: false, maxLength: 20, minLength: 6 }
    },
    {
      name: 'password-confirmation',
      format: 'password',
      placeholder: 'Confirm New Password',
      validations: { required: true, allowEmpty: false, maxLength: 20, minLength: 6, confirmationOf: 'newPassword' }
    }
  ]
};
