// @flow

import React from 'react';
import { Tabs, Tab } from 'react-bootstrap';
import _ from 'lodash/core';

export default class TabContainer extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      key: 1,
      sharedData: {},
      hidenTabs: []
    };

    this.handleSelect = this.handleSelect.bind(this);
    this.shareData = this.shareData.bind(this);
    this.hideTab = this.hideTab.bind(this);
    this.unHideTab = this.unHideTab.bind(this);
  }

  componentWillMount() {
    // check if router passed a arr keyNum to use as initial Tab
    if (this.props.location.state && this.props.location.state.keyNum) {
      let key = this.props.location.state.keyNum;
      this.setState({key: key});
    }
  }

  handleSelect(key) {
    this.setState({key: key});
  }

  // for updating sharedData
  shareData(data) {
    // add the provided data to our sharedData
    let sharedData = Object.assign({}, this.state.sharedData, data);
    this.setState({sharedData: sharedData});
  }

  hideTab(scId) {
    let hidenTabs = this.state.hidenTabs;
    if (-1 == hidenTabs.indexOf(scId)) {
      hidenTabs.push(scId);
      this.setState({hidenTabs: hidenTabs});
    }
  }

  unHideTab(scId) {
    let hidenTabs = this.state.hidenTabs;
    if ('ALL' === scId) {
      this.setState({hidenTabs: []});
    }
    else {
      let index = hidenTabs.indexOf(scId);
      if (-1 !== index) {
        hidenTabs.splice(index, 1);
        this.setState({hidenTabs: hidenTabs});
      }
    }
  }

  render() {
    const self = this;
    const getComponent = this.props.getComponent;
    const {content, ...props} = this.props;
    // make children Tabs
    let children = [], at = 0;
    _.forEach(content.scContent.children, function(val) {
      // skip hidden tabs
      if (-1 !== self.state.hidenTabs.indexOf(val.scId)) return;
      // increase tab id
      at++;
      // check if we should not display tabs when id param is missing
      if (0 < children.length && content.scContent.noIdHideAllButFirst
        && (!props.params || !props.params.id)) {
        // stop loop
        return false;
      }
      let NodeComponent = getComponent(val.scType, 'Html');
      children.push(
        <Tab key={`tab-${val.scId}`} eventKey={at} title={val.scTitle}>
          <NodeComponent content={val} parentTabs={self} {...props} />
      </Tab>
      );
    });
    return (
      <Tabs id={`tabs-${this.props.content.scId}`} activeKey={this.state.key}
        onSelect={this.handleSelect} {...content.props}>
        {children}
      </Tabs>
    );
  }
}
