// @flow

// this is a higher-order component (HOC) that wraps Route App list of pages

import React, { Component } from 'react';
import _ from 'lodash/core';
import {
  preloadContents, fetchPages, refreshPage
} from './../redux/actions/pages';
import { fetchProfile, updateSiteData } from './../redux/actions/zlSite';
var axios = require('axios');
var events = require('../../../../../socket/actions');
var bowser = require('bowser');

export default function appSubscription(WrappedComponent: any) {
  return class extends Component {
    constructor(props) {
      super(props);

      let self = this;

      if (process.env.APP_ENV === 'browser') {
        // add css classes for browser
        if (bowser.name) {$('html').addClass(`is${bowser.name.replace(' ', '')}`);}
        if (bowser.mobile) {$('html').addClass('isMobile');}
        else if (bowser.tablet) {$('html').addClass('isTablet');}
        // may not really be desktop, just means mobile/tablet could not be determin
        else {$('html').addClass('isDesktop');}

        // configuring default options for Messenger
        Messenger.options = {
          theme: 'flat'
        };

        // init ajax defaults
        this.setAjaxDefault(props.zlSite);

        // listen for updates on main menu
        socket.on(events.MENU_UPDATED, (data) => {
          props.dispatch(fetchPages(props.zlSite.zsId));
        });
        // listen for updates on page
        socket.on(events.PAGE_UPDATED, (data) => {
          props.dispatch(refreshPage(data));
        });
        // listen for updates on model data for page content
        socket.on(events.EDIT_DATA_OF, (data) => {
          let updPg = _.filter(props.pages, function(o) {
            return (o.scsOptions && data == o.scsOptions.hasDataOf)
          });
          _.forEach(updPg, function(pg) {
            self.props.dispatch(refreshPage(pg.scsId));
          });
        });
        // listen for update to user
        if (props.zlSite.user) {
          socket.on(events.USER_UPDATED, (usrId) => {
            if (usrId === props.zlSite.user.usrId) {
              props.dispatch(fetchProfile(usrId));
            }
          });
        }
      }

      // check for theme specific constructors and run them
      _.forEach(props.loadedComponents, function(val) {
        if (!_.isUndefined(val.appContructor)) {
          val.appContructor(props);
        }
      });

      // keep list of pages we tried to fetch
      this.prefetchTried = [];

      this.prefetchPage = this.prefetchPage.bind(this);
      this.updateActiveVariant = this.updateActiveVariant.bind(this);
    }

    setAjaxDefault(zlSite) {
      // Add axios request interceptor
      axios.interceptors.request.use(function (config) {
        if (config.url.startsWith('/api/')) {
          // when request is to local zealder api
          config.headers.common['X-Access-Token'] = zlSite.accessToken;
          config.headers.common['X-Base-Url-Path'] = zlSite.baseUrlPath;
          if (zlSite.user && zlSite.user.simulating) {
            config.headers.common['X-User-Simulating'] = true;
          }
        }
        return config;
      });

      /* add default headers to every request.
      NOTE: deprecated; use axios instead */
      $.ajaxSetup({
          // accessToken
          headers: {
            'X-Access-Token': zlSite.accessToken,
            'X-Base-Url-Path': zlSite.baseUrlPath,
            'X-User-Simulating': (zlSite.user && zlSite.user.simulating) ? 1:''
          }
      });
    }

    componentDidMount() {
      /* for some reason prefetchPage will cause render of components
      (probably shouldn't use any redux props this high up) and this may not be
      ideal in some cases (i.e. when a dialog is open, it will loose focus).
      Any function that want to pause it will simply push their reason/id to
      list and removeto unpause */
      window.pausePrefetchPageFor = [];

      // prefetch top page/state contents
      this.prefetchPage(3); // a few for now until we have cache and request for many simultanously

      // caller for setting variant in experiment
      window.zlUpdateActiveVariant = this.updateActiveVariant;
    }

    updateActiveVariant(variantId) {
      this.props.dispatch(updateSiteData({variantActive: variantId}));
    }

    componentWillReceiveProps(nextProps) {
      if (nextProps.zlSite.accessToken !== this.props.zlSite.accessToken) {
        // when auth token change reset ajax defaults
        this.setAjaxDefault(nextProps.zlSite);
        // since they just signed in, also get profile
        this.props.dispatch(fetchProfile(nextProps.zlSite.user.usrId));
      }
    }

    prefetchPage(numLeft) {
      if (numLeft < 1) {return;}
      let self = this;
      _.delay(function() {
        // wait for all other process to finish
        if (Pace.running) {
          Pace.once('hide', () => {
            self.prefetchPage(numLeft);
          });
        }
        else if (0 < window.pausePrefetchPageFor.length) {
          self.prefetchPage(numLeft);
        }
        else {
          // fetch next priority page that hasn't been fetched
          // TODO: make better sort. As-is doesn't really target top pages
          let fetchPage = _.find(
            _.sortBy(self.props.pages, 'scsOrder'),
            function(o) {
              return (
                (!o.contents || o.contents.length === 0) && !o['lastUpdated']
                && 'link' !== o.scsBase && 'framed link' !== o.scsBase
                && 'menu label' !== o.scsBase
                && (!o.scsOptions || !o.scsOptions.dontFetchContentOnLoad)
                && -1 == self.prefetchTried.indexOf(o.scsId)
              );
            });
          if (!_.isEmpty(fetchPage)) {
            self.props.dispatch(preloadContents(fetchPage.scsId));
            self.prefetchPage((numLeft-1));
            self.prefetchTried.push(fetchPage.scsId);
          }
        }
      }, 1000);
    }

    render() {
      return <WrappedComponent {...this.props} />;
    }
  }
}
