// @flow

import "babel-polyfill";
import React from 'react';
import ReactDOM from 'react-dom';
import { Router, browserHistory } from 'react-router';
import routes from './routes';
import { Provider } from 'react-redux';
import { createStore, applyMiddleware, compose } from 'redux';
import reducers from './redux/reducers';
import thunk from 'redux-thunk';
// for revealing animation on components
import {WOW} from 'wowjs/dist/wow';
window.WOW = WOW;

// don't show loader for socket requests
Pace.options.ajax.trackWebSockets = false;

const middlewares = [thunk];

if (process.env.NODE_ENV == 'development') {
  const createLogger = require(`redux-logger`).createLogger;
  const logger = createLogger();
  middlewares.push(logger);
}

//
/**
 * make routing available global so even html from db can route pages
 * also allows component to not create their own browserHistory which
 * may not work (@see: https://github.com/ReactTraining/react-router/issues/4640)
 */
window.routeToPage = function (path, query = null, state = null) {
  // default will use url (can also include query params)
  var pathOrLoc = path;
  // check if Loc object should be passed instead
  if (query || state) {
    pathOrLoc = { pathname: path, query: query, state: state}
  }
  browserHistory.push(pathOrLoc);
  // prevent browser link
  return false;
}
window.browserHistory = browserHistory;

// initial data
var zlData = document.getElementById('zlData');
zlData = zlData ? JSON.parse(zlData.textContent) : '';

// generic error message (unknown error)
window.defaultErrorMsg = 'Could not process your request. Please try again later.';

let store = compose(applyMiddleware(...middlewares))(createStore)(reducers, zlData);

ReactDOM.render(
  <Provider store={store}>
    <Router history={browserHistory}>
      {routes(zlData)}
    </Router>
  </Provider>,
  document.getElementById('app-container')
);
