// @flow

import _ from 'lodash/core';

export const SELECT_PAGE = 'SELECT_PAGE';
export const INVALIDATE_PAGE = 'INVALIDATE_PAGE';
export const RECEIVE_PAGES = 'RECEIVE_PAGES';
export const REQUEST_CONTENTS = 'REQUEST_CONTENTS';
export const RECEIVE_CONTENTS = 'RECEIVE_CONTENTS';
export const RECEIVE_CONTENTS_PRE = 'RECEIVE_CONTENTS_PRE';

export function selectPage(scsId) {
  return {
    type: SELECT_PAGE,
    scsId
  }
}

// fetch pages (for menu/navigation)
export function fetchPages(zsId) {
  return dispatch => {
    return $.get( `/api/zlSiteContentSets/menupages`, {zsId: zsId })
      .done(data => dispatch(receivePages(data)))
      // TODO: add error handler
  }
}

export function receivePages(pages) {
  return {
    type: RECEIVE_PAGES,
    pages: pages.data
  }
}

export function requestContents(scsId) {
  return {
    type: REQUEST_CONTENTS,
    scsId
  }
}

// when receive list of contents for a page
export function receiveContents(scsId, json) {
  return {
    type: RECEIVE_CONTENTS,
    scsId,
    contents: json.data,
    receivedAt: Date.now()
  }
}

// when receive list of preloaded contents for a page
export function receiveContentsPre(scsId, json) {
  return {
    type: RECEIVE_CONTENTS_PRE,
    scsId,
    contents: json.data
  }
}

// fetch all contents for a page
export function fetchContents(scsId) {
  return dispatch => {
    dispatch(requestContents(scsId));
    return $.get( `/api/zlSiteContentIns/contentsToDisplay?scsId=${scsId}`)
      .done(data => dispatch(receiveContents(scsId, data)))
      // TODO: should show Error instead of empty content
      .fail(() => dispatch(receiveContents(scsId, {})));
  }
}

// for preloading content (saves seperate from pages to not cause render)
export function preloadContents(scsId) {
  return dispatch => {
    return $.get( `/api/zlSiteContentIns/contentsToDisplay?scsId=${scsId}`)
      .done(data => {
        if (data && !_.isEmpty(data.data)) dispatch(receiveContentsPre(scsId, data))
      });
  }
}

// reloads a page if we already have it
export function refreshPage(scsId) {
  return (dispatch, getState) => {
    if (getState().pages[scsId].contents.length > 0) {
      return dispatch(fetchContents(scsId));
    }
  }
}

// determine if contents for a page already exist
function shouldFetchContents(state, scsId) {
  const page = state.pages[scsId];
  if (page.contents.length === 0) {
    return true
  } else if (page.isFetching) {
    return false
  } else {
    return page.didInvalidate
  }
}

// fetch content for a page if it hasn't been loaded
export function fetchContentsIfNeeded(scsId) {
  return (dispatch, getState) => {
    if (shouldFetchContents(getState(), scsId)) {
      // check if was preloaded
      if (getState().pagesPreload[scsId]) {
        return dispatch(receiveContents(scsId, {data: getState().pagesPreload[scsId]}));
      }
      else return dispatch(fetchContents(scsId));
    }
  }
}
