// @flow

// reducers for Pages (all content sets (page, categories, etc))

import _ from 'lodash/core';

import {
  SELECT_PAGE, INVALIDATE_PAGE, RECEIVE_PAGES, RECEIVE_CONTENTS_PRE,
  REQUEST_CONTENTS, RECEIVE_CONTENTS
} from './../actions/pages'

// manage current selected page id
export function currentPage(state = 0, action) {
  switch (action.type) {
  case SELECT_PAGE:
    return action.scsId
  default:
    return state
  }
}

// list of site content sets
export function pages(state = [], action) {
  switch (action.type) {
    case REQUEST_CONTENTS:
      return Object.assign({}, state, {
        [action.scsId]: Object.assign({}, state[action.scsId], {
          isFetching: true,
          didInvalidate: false
        })
      })
    case RECEIVE_CONTENTS:
      let contents = [];
      _.forEach(action.contents, function(val) {
        if (!val) return;
        contents.push(val.scId);
      });
      return Object.assign({}, state, {
        [action.scsId]: Object.assign({}, state[action.scsId], {
          isFetching: false,
          didInvalidate: false,
          contents: contents,
          lastUpdated: action.receivedAt
        })
      })
    case INVALIDATE_PAGE:
      return Object.assign({}, state, {
        [action.scsId]: Object.assign({}, state[action.scsId], {
          didInvalidate: true
        })
      })
    case RECEIVE_PAGES:
      // keep list of content ids
      _.forEach(action.pages, function(page, scsId) {
        if (_.isEmpty(page.contents) && state[scsId]) {
          page.contents = _.clone(state[scsId].contents)
        }
        if ((!state[scsId] || state[scsId].isNew ||
          page.scsSlug != state[scsId].scsSlug ||
          page.scsParent != state[scsId].scsParent) && 'link' !== page.scsBase) {
          /* if a new page was added or the path changed, we force a full
          url load of new page since
          React Router 3 doesn't support hot updating routes. I believe
          upgrading to React Router 4 will solve this
          @see: https://github.com/ReactTraining/react-router/issues/2182 */
          page.isNew = true;
          page.scsBase = 'link';
          page.scsOptions = {Url: page.path, UseBasePath: true}
        }
      });
      return action.pages;
    default:
      return state
  }
}

// list of site content for content sets
export function siteContents(state = { }, action) {
  switch (action.type) {
    case RECEIVE_CONTENTS:
      let contents = {};
      _.forEach(action.contents, function(val) {
        if (!val) return;
        contents[val.scId] = val;
      });
      return Object.assign({}, state, contents)
    default:
      return state
  }
}

// list of preloaded pages
export function pagesPreload(state = { }, action) {
  switch (action.type) {
    case RECEIVE_CONTENTS_PRE:
      return Object.assign({}, state, {[action.scsId]: action.contents})
    case RECEIVE_CONTENTS:
      // remove from preload since its loaded now
      let copy = Object.assign({}, state)
      delete copy[action.scsId]
      return copy
    default:
      return state
  }
}
