// @flow

import React from 'react';
import _ from 'lodash/core';

export default class PgStandard extends React.Component {
  componentDidMount() {
    const { page, contentFillsWindow, location, gtags } = this.props;

    this.doWow();

    // scroll to top of page on each page load since router doesn't do so
    window.scrollTo(0, 0);

    // track page if using "Legacy" (ga.js) analytics
    if ('_gaq' in window) {
      // first check if page provide own track event
      if (page._gaq) _gaq.push(page._gaq);
      // send pageview
      _gaq.push(['_trackPageview', location.pathname ]);
    }

    // Google analytic
    if (gtags && 'gtag' in window) {
      _.forEach(gtags, (val, id) => {
        let conf = Object.assign({'page_path': location.pathname }, val);
        gtag('config', id, conf);
      });
    }

    if (typeof dataLayer !== "undefined") {
      // initiate Google optimize
      dataLayer.push({'event': 'optimize.activate'});
      // changes made directly in Google Optimize (not recommended/supported)
      // need a slight delay because React may not have finish loading
      setTimeout(function() {
        dataLayer.push({'event': 'optimize.activate'});
      }, 100);
    }

    // add page name and component class
    $('html').addClass(`pg-comp-${page.scsBase.replace(/\s/g, '-')}`);
    $('html').addClass(`page-${page.scsName.replace(/\s/g, '-')}`);

    // add/remove content-fills-window class
    if (contentFillsWindow) $('html').addClass('content-fills-window');
    else $('html').removeClass('content-fills-window');
  }

  componentDidUpdate() {
    const { page, htmlMeta } = this.props;

    this.doWow();

    // metaData
    let meta = (htmlMeta) ? htmlMeta : page.scsHtmlMeta;
    // correct page title
    let pgTitle = '';
    if (meta && meta.title) {
      pgTitle = meta.title;
    }
    else if (page.scsName) {
      pgTitle = page.scsName;
    }
    $(document).prop('title', pgTitle);
  }

  componentWillUnmount() {
    const { page } = this.props;
    // remove page name class
    $('html').removeClass(`pg-comp-${page.scsBase.replace(/\s/g, '-')}`);
    $('html').removeClass(`page-${page.scsName.replace(/\s/g, '-')}`);
  }

  render() {
    const { getComponent, zsDefaultContent, contentFillsWindow, location } = this.props;
    const props = this.props;

    let bodyNodes = <h1>Oops, this page is empty</h1>;

    if (props.isFetching) {
      // display loader while fetching
      let Loader = getComponent('Loader', '');
      bodyNodes = <Loader />;
    }

    // build nodes to display on this page
    let headerNodes = [], footerNodes = [];
    if (this.props.contentList && this.props.contentList.length > 0) {
      /* ContentWrapper: used by themes to wrap all top level body content for
      pages with a layout (e.g., panels for admin theme, layout container, etc) */
      let ContentWrapper = getComponent('ContentWrapper', '');
      // reset as array
      bodyNodes = [];
      _.forEach(this.props.contentList, function(val) {
        if (!val || !val.scType) return;

        let NodeComponent = getComponent(val.scType, 'Html');

        if ('header' == val.scPlacement) {
          headerNodes.push(
            <NodeComponent key={val.scId} content={val} getComponent={getComponent}
              contentFillsWindow={contentFillsWindow} />
          );
        }
        else if ('body' == val.scPlacement && !props.bodyContent) {
          // wrap content
          let node = (
            <ContentWrapper key={val.scId} content={val} {...props} >
              <NodeComponent content={val} {...props} />
            </ContentWrapper>
          );
          bodyNodes.push(node);
        }
        else if ('footer' == val.scPlacement && !contentFillsWindow) {
          footerNodes.push(
            <NodeComponent key={val.scId} content={val} {...props} />
          );
        }
      });
    }

    if (props.bodyContent) {
      // when bodyContent is passed as prop, it should be the only body content
      bodyNodes = this.props.bodyContent;
    }

    if (_.isEmpty(headerNodes) && zsDefaultContent && zsDefaultContent.header) {
      // this page has no assign header, use default
      let HeaderComponent = getComponent(zsDefaultContent.header.scType, 'Header');
      headerNodes.push(
        <HeaderComponent key="defaultHeader" content={zsDefaultContent.header}
          getComponent={getComponent} contentFillsWindow={contentFillsWindow} />
      );
    }
    if (_.isEmpty(footerNodes) && zsDefaultContent && zsDefaultContent.footer &&
      !contentFillsWindow) {
      // this page has no assign footer, use default
      let FooterComponent = getComponent(zsDefaultContent.footer.scType, 'Footer');
      footerNodes.push(
        <FooterComponent key="defaultFooter" getComponent={getComponent}
          content={zsDefaultContent.footer} />
      );
    }

    // load reusable modal that any page can call up
    let GlobalModal = getComponent('GlobalModal', '');

    return (
      <div id='page-wrap-inner'>
        <GlobalModal query={location.query} />
        {0 < headerNodes.length &&
          <header className="app-header">{headerNodes}</header>
        }
        <div id='page-body'>{bodyNodes}</div>
        {0 < footerNodes.length &&
          <footer className="app-footer">{footerNodes}</footer>
        }
      </div>
    )
  }

  // TODO: need to find better way to this; we do this because when React
  // draws late, component remains hidden (ie when ajax call re-renders)
  doWow() {
    // reveal any animation.css
    if (window.pgWow) window.pgWow.sync();
    else {
      var wowChecks = 0;
      var wowLoader = window.setInterval(() => {
        wowChecks++;
        if (wowChecks > 50) {
          // checked too many times; maybe no wow item on page
          window.clearTimeout(wowLoader);
        }
        window.pgWow = new window.WOW({live: true, callback: () => {
          window.clearTimeout(wowLoader);
        }});
        window.pgWow.init();
      }, 200);
    }
  }
}
